/**********************************************************************
 *
 *       util.c  -  General support routines for uhfa.c
 *                        Version 2.02
 *
 * NOTICE: Copyright (C) 2000 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm  and,
 * http://www.ao.com/~gnome/
 *
 * Transferred form UHFA.C 22 Nov 00 -  RSL
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <graphics.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <dos.h>
#include "util.h"
#include "moonsun.h"
#include "uhfa.h"
#include "lbox.h"
#include "u_code.h"
#include "async.h"
#include "gifsave.h"
#include "umatrix.h"

void delayu(unsigned int);
void do_warm_boot(void);	/* Reset dsp & hardware */
/* void do_gif_file(void); */
void print_big(char, int, int);
void print_super_row(char, int, int);
void print_medium(char, int);
void print_medium_1(char, int);
void print_medium_2(char, int);
void dsp_cmd(int, int, int, int, int);
void show_last_cmd(char* ct);
void show_help(void);
void ctrlc_handler(int sig);
void fpe_handler(int sig);
char *left_justify(char *str1, char *str2, int len2, char fill);
void license_info(void);
void license(void);
void warranty(void);
void quit(void);

#if 0
/*   For use when allowable */
/* do_gif_file()  -  Records entire graphics screen in a file.
 * The file has a name 'Umddhhmm.GIF' where m is the month (1,C),
 * hhmm is the hour and minute.
 * The screen must be in VGA 640x480, 16 colors.
 * Based on S. H. Huseby (gifsave.c)
 */
void do_gif_file(void)
	{
	#define NUMCOLORS         16  /* Number of different colors */
	#define BITS_PR_PRIM_COLOR 2  /* Two bits per primary color */

	int gpixel(int x, int y);

	int q,                      /* Counter */
	    color,                  /* Temporary color value */
	    red[NUMCOLORS],         /* Red component for each color */
	    green[NUMCOLORS],       /* Green component for each color */
	    blue[NUMCOLORS];        /* Blue component for each color */
	struct palettetype pal;
	int ic;
	int itime;
	char ct[15];
	time_t t0;
	struct tm *tgp;

	if (!screen_save) return;

	setviewport(0,80,9,90,DO_CLIP);
	clearviewport();
	setcolor(c_text_note);
	settextstyle(0, 0, 1);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(0,1);
	outtext("*");

	cc[0]='U';

	if (ms_available)
	    t0=update_clock(1)+dsp_clock_base;
	else
	    (void)time(&t0);
	if (time_type)
	    tgp = localtime(&t0);	/* time_type==1 for local */
	else
	    tgp = gmtime(&t0);		/*          ==0 for gmt */

	itoa( (1+tgp->tm_mon), ct, 13);   /* Jan=1,..., Dec=C */
	cc[1]=ct[0];
	itoa(tgp->tm_mday, ct, 10);
	if (tgp->tm_mday < 10)
	    {
	    cc[2]='0';
	    cc[3]=ct[0];
	    }
	else
	    {
	    cc[2]=ct[0];
	    cc[3]=ct[1];
	    }
	/* 100*hour+min will be between 0 and 2359. Need to trap the leading
	 * zero cases to prevent a blank from getting to file name.
	 */
	itime=100*tgp->tm_hour + tgp->tm_min;
	itoa( itime, ct, 10);
	if (tgp->tm_hour < 10)
	    {
	    cc[4]='0';		/* Less than 0959, so lead with '0' */
	    if (itime>=100)	/* 100 to 999 */
	      {cc[5]=ct[0];  cc[6]=ct[1];  cc[7]=ct[2];}
	    else if (itime>=10)	/* 10 to 99 */
	      {cc[5]='0';  cc[6]=ct[0];  cc[7]=ct[1];}
	    else		/* 0 to 9 */
	      {cc[5]='0';  cc[6]='0';  cc[7]=ct[0];}
	    }
	else
	    {
	    cc[4]=ct[0]; cc[5]=ct[1]; cc[6]=ct[2]; cc[7]=ct[3];
	    }
	cc[8]='.'; cc[9]='G'; cc[10]='I'; cc[11]='F'; cc[12]=0;

	setviewport(0, 0, 639, 479, 1);	/* Needed to start gif right */

	/* Get the color palette, and extract the red, green and blue
 	*  components for each color. In the EGA/VGA palette, colors are
	*  stored as bits in bytes:
	*      00rgbRGB
	*  where r is low intensity red, R is high intensity red, etc.
	*  We shift the bits in place like
	*      000000Rr
	*  for each component					*/
	getpalette(&pal);
	for (q = 0; q < NUMCOLORS; q++)
	    {
            color = pal.colors[q];
            red[q]   = ((color & 4) >> 1) | ((color & 32) >> 5);
            green[q] = ((color & 2) >> 0) | ((color & 16) >> 4);
            blue[q]  = ((color & 1) << 1) | ((color & 8) >> 3);
	    }

	GIF_Create(cc, 640, 480, NUMCOLORS, BITS_PR_PRIM_COLOR);

	/*  Set each color according to the values extracted from
	 *  the palette    */
	for (q = 0; q < NUMCOLORS; q++)
	GIF_SetColor(q, red[q], green[q], blue[q]);

	/*  Store the entire screen as an image using the user defined
	 *  callback function gpixel() to get pixel values from the screen */
	GIF_CompressImage(0, 0, -1, -1, gpixel);

	GIF_Close();

	setviewport(0,80,9,90,DO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	}

int gpixel(int x, int y)
	{
	return getpixel(x, y);
	}
#endif


/* warm_boot()  -  This is a full reset of the DSP.
 */
void do_warm_boot(void)
	{
	long a, b;

	delay(30);			/* Flush buffers, uarts etc. */

	ASYNCOUT(ESC_CHAR);  ASYNCOUT('R');	/* Fresh start */
	ASYNCOUT(32); ASYNCOUT(2);	/* 32 is immediate fresh start */
	ASYNCOUT(3); ASYNCOUT(4);

	delay(200);			/* Wait for reset to finish */

	/* Now reprogram dsp to current states */
	ASYNCOUT(ESC_CHAR);  ASYNCOUT('J');
	ASYNCOUT(input_if_af);
	ASYNCOUT(0); ASYNCOUT(3); ASYNCOUT(4);   /* Audio to L & R channels */

	set_window();
	set_mode(0);
	set_afgain(0);
	set_rfgain();
	set_agc(agc_decay);
	set_8_16(9);			/* 9==>program to current value */
	set_affilt_sw();
	set_affilt_select();
	set_denoise();
	set_delay_right(delay_right);
	new_freq();
	set_display_width();
	}

/* print_big(ch, pos, color) prints a double size character in the top
 * box.  The position can be specified as 0 to 30, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * Available in waterfall configuration, only. No backspace.
 */
void print_big(char ch, int pos, int color)
	{static int print_pos=0;
	 static char c[31];
	 static char c1[2];
	 int i;

	 if (pos==-1)			/* Initialize data */
	    {for(i=0; i<30; i++)  c[i]=32;
	     c[30]=0;
	     print_pos=0;
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==30)		/* In scroll position */
		{for (i=0; i<29; i++)  c[i]=c[i+1];
		 c[29]=ch;
		}
	     else			/* Not yet scrolling */
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;
	    c1[0]=ch; c1[1]=0;
	    setviewport(132+16*pos, 13, 148+16*pos, 32, DO_CLIP);
	    clearviewport();
	    setcolor(color);
	    settextstyle(0, 0, 2);
	    settextjustify(0,2);
	    moveto(0,2);
	    outtext(c1);
	    setcolor(c_text_bold);
	    return;
	    }
	 setviewport(129, 13, 638, 32, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 2);
	 settextjustify(0,2);
	 moveto(3,2);
	 outtext(c);
	}

/* print_super_row(ch, pos, color) prints a in the top box, centered over
 * the character from print_big(...)
 * The position can be specified as 0 to 30. If pos=-1 then initialize.
 * Available in waterfall configuration, only. No backspace.
 */
void print_super_row(char ch, int pos, int color)
	{
	static char c1[2];

	if (pos==-1)			/* Initialize data */
	    {
	    c1[0]=0; c1[1]=0;
	    setviewport(132, 2, 628, 12, DO_CLIP);
	    clearviewport();
	    }
	else
	    {
	    c1[0]=ch; c1[1]=0;
	    setviewport(135+16*pos, 2, 146+16*pos, 12, DO_CLIP);
	    clearviewport();
	    setcolor(color);
	    settextstyle(0, 0, 1);
	    settextjustify(0,2);
	    moveto(0,1);
	    outtext(c1);
	    setcolor(c_text_bold);
	    return;
	    }
	}



/* print_medium(ch, pos) prints a standard size character in the top
 * box.  The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 */
void print_medium(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[3*LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<3*LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[3*LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==254)		/* Code for backspace */
	    {
	    if (print_pos>0)
		{
		c[--print_pos] = 32;		/* Erase last char */
		}
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==3*LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<3*LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[3*LINE_LEN-2]=ch;
		}
	     else
		{
		c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;			/* Random access to pos */
	    }
	 if (waterfall)
	     setviewport(129, 1, 638, 33, DO_CLIP);
	 else
	     setviewport(X2L+1, Y2T+59, X2R-1, Y2T+98, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);
	 moveto(3, 2);
	 save_ch = c[LINE_LEN];  c[LINE_LEN] = 0;
	 outtext(c);
	 c[LINE_LEN] = save_ch;
	 save_ch = c[2*LINE_LEN];  c[2*LINE_LEN] = 0;
	 moveto(3, 12);
	 outtext(&c[LINE_LEN]);
	 c[2*LINE_LEN] = save_ch;
	 moveto(3, 21);
	 outtext(&c[2*LINE_LEN]);
	}

/* print_medium_1(ch, pos) prints a standard size character in the top
 * box.  Single line at top ONLY.
 * The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * Also responds to pos=254 and does a backspace.
 * Available in waterfall configuration, only.
 */
void print_medium_1(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==254)		/* Code for backspace */
	    {
	    if (print_pos>0)
	    c[--print_pos] = 32;	/* Erase last char */
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[LINE_LEN-2]=ch;
		}
	     else
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;			/* Random access to position */
	    }

	 setviewport(129, 1, 638, 11, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);
	 moveto(3, 2);
	 outtext(c);
	}

/* print_medium_2(ch, pos) prints a standard size character in the top
 * box.  This one prints 2 line, in the bottom 2/3 of the box.
 * The position can be specified by pos, or by setting pos=255
 * it will be printed in the next available position.  Scrolls to the left
 * when full, Times Sq style (if pos=255.)  If pos=-1 then initialize.
 * No backspace, available in waterfall configuration, only.
 */
void print_medium_2(char ch, int pos)
	{
	 #define LINE_LEN 60
	 static int print_pos;
	 static char c[2*LINE_LEN];
	 char save_ch;
	 int i;

	 if (pos==-1)			/* Code for initialize buffer */
	    {for(i=0; i<2*LINE_LEN; i++)  c[i]=32;  /* Blank */
	     c[2*LINE_LEN-1]=0;
	     print_pos=0;		/* Start at first position */
	    }
	 else if(pos==255)		/* In scroll mode */
	    {if (print_pos==2*LINE_LEN-1)	/* In scroll position */
		{for (i=0; i<2*LINE_LEN-2; i++)  c[i]=c[i+1];
		 c[2*LINE_LEN-2]=ch;
		}
	     else
		{c[print_pos++]=ch;
		}
	    }
	 else
	    {
	    c[pos]=ch;
	    }

	 setviewport(129, 12, 638, 33, DO_CLIP);
	 clearviewport();
	 settextstyle(0, 0, 1);
	 settextjustify(LEFT_TEXT, TOP_TEXT);

	 moveto(3, 2);
	 save_ch = c[LINE_LEN];  c[LINE_LEN] = 0;
	 outtext(c);
	 c[LINE_LEN] = save_ch;

	 moveto(3, 12);
	 outtext(&c[LINE_LEN]);
	}

void dsp_cmd(int b1, int b2, int b3, int b4, int b5)
	{
	ASYNCOUT(ESC_CHAR);
	ASYNCOUT(b1 & 0XFF);
	ASYNCOUT(b2 & 0XFF);
	ASYNCOUT(b3 & 0XFF);
	ASYNCOUT(b4 & 0XFF);
	ASYNCOUT(b5 & 0XFF);
	}

void show_last_cmd(char* ct)
	{
	setviewport(40, 470, 89, 479, DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(25, 0);
	setcolor(c_text);
	outtext(ct);
	setcolor(c_text_bold);
	}

void show_help(void)
	{
	char cc1[7], cc2[33];
	int i;

	setcolor(c_text_bold);
	settextjustify(0,2);		/* Left and top */
	/* Works out to be 224 pixels high: */
	setviewport(X2L+1, Y2T+96, X2R-1, Y2B-12, DO_CLIP);
	getimage(0,  0, 510, 160, big_area_ptr1);
	getimage(0, 161, 510, 224, big_area_ptr2);
	clearviewport();
	line(1, 0, 509, 0);
	line(1, 223, 509, 223);
 /* 63 characters:
          123456789012345678901234567890|23456789012345678901234567890123 */
 moveto(3, 3);			/* Line 1-1 */
 outtext("             DSP-10 KEYBOARD COMMANDS - Screen 1");
 moveto(3, 13);			/* Line 1-2 */
 outtext("       FREQUENCY                                GAIN");
 moveto(3, 23);			/* Line 1-3 */
 outtext("1 Hz Down    Ctrl-Alt-Sh F9   |   AF Gain Down 2dB           F7");
 moveto(3, 33);			/* Line 1-4 */
 outtext("1 Hz Up      Ctrl-Alt-Sh F10  |   AF Gain Up 2 dB            F8");
 moveto(3, 43);			/* Line 1-5 */
 outtext("10 Hz Down               F9   |   RF Gain Down 6 dB       Sh F7");
 moveto(3, 53);			/* Line 1-6 */
 outtext("10 Hz Up                 F10  |   RF Gain Up 6 dB         Sh F8");
 moveto(3, 63);			/* Line 1-7 */
 outtext("100 Hz Down        Shift F9   |   Mike Gain Down 1 dB Alt-Sh F7");
 moveto(3, 73);			/* Line 1-8 */
 outtext("100 Hz Up          Shift F10  |   Mike Gain Up 1 dB   Alt-Sh F8");
 moveto(3, 83);			/* Line 1-9 */
 outtext("1 kHz Down           Alt F9   |   Require Scroll Lock On:");
 moveto(3, 93);			/* Line 1-10 */
 outtext("1 kHz Up             Alt F10  |   AGC Release Faster     Scrl G");
 moveto(3, 103);		/* Line 1-11 */
 outtext("10 kHz Down         Ctrl F9   |   AGC Release Slower     Scrl H");
 moveto(3, 113);		/* Line 1-12 */
 outtext("10 kHz Up           Ctrl F10  |   CW Sidetone Down 1 dB  Scrl S");
 moveto(3,123);			/* Line 1-13 */
 outtext("100 kHz Down    Ctrl-Alt F9   |   CW Sidetone Up 1 dB    Scrl D");
 moveto(3,133);			/* Line 1-14 */
 outtext("100 kHz Up      Ctrl-Alt F10  |   CW Offset Down         Scrl C");
 moveto(3,143);			/* Line 1-15 */
 outtext("1 MHz Down       Ctrl-Sh F9   |   CW Offset Up           Scrl V");
 moveto(3, 153);		/* Line 1-16 */
 outtext("RIT 10Hz Down     Alt R or r  |       TRANSMIT/RECEIVE");
 moveto(3, 163);		/* Line 1-17 */
 outtext("RIT 10Hz Up       Alt T or t  |   T/R Toggle               Home");
 moveto(3, 173);		/* Line 1-18 */
 outtext("RIT On/Off        Alt O or o  |          AF FILTERS");
 moveto(3, 183);		/* Line 1-19 */
 outtext("RIT Zero          Alt Z or z  |   Filters On/Off             F3");
 moveto(3, 193);		/* Line 1-20 */
 outtext("RIT 1Hz Up   Ctrl-Alt R or r  |   Filter Change              F4");
 moveto(3, 203);		/* Line 1-21 */
 outtext("RIT 1Hz Dwn  Ctrl-Alt T or t  |   DeNoise On/Off          Sh F3");
 moveto(3, 213);		/* Line 1-22 */
 outtext("                              |   De-Noise Gain    Ctrl F3 & F4");

	for(;;)			/* Wait for keyboard hit */
	    if ( kbhit() )
		{ _bios_keybrd(_KEYBRD_READ);  break; }
	clearviewport();
	line(1, 0, 509, 0);
	line(1, 223, 509, 223);
 /* 63 characters:
          123456789012345678901234567890|23456789012345678901234567890123 */
 moveto(3, 3);			/* Line 2-1 */
 outtext("             DSP-10 KEYBOARD COMMANDS - Screen 2");
 moveto(3, 13);			/* Line 2-2 */
 outtext("          FM                                      OTHER");
 moveto(3, 23);			/* Line 2-3 */
 outtext("Squelch Down     Alt S or s   |   Spectral Dsply Ave Dwn Alt F3");
 moveto(3, 33);			/* Line 2-4 */
 outtext("Squelch Up       Alt D or d   |   Spectral Dsply Ave Up  Alt F4");
 moveto(3, 43);			/* Line 2-5 */
 outtext("                              |   Norm Spectrum Fast Alt Q or q");
 moveto(3, 53);			/* Line 2-6 */
 outtext("          CW                  |   Norm Spectr 10 min Alt P or p");
 moveto(3, 63);			/* Line 2-7 */
 outtext("Alphanumeric  Any key in CW   |   Spectr Norm On/Off Alt N or n");
 moveto(3, 73);			/* Line 2-8 */
 outtext("Delete Last Ch    Backspace   |   Color Display AGC  Alt C or c");
 moveto(3, 83);			/* Line 2-9 */
 outtext("Delete Buffer        Sh DEL   |   IF bandwidth W/N        Alt $");
 moveto(3, 93);			/* Line 2-10 */
 outtext("CW Speed Slower      Alt F7   |   Spectr Disp dB/div      Alt 1");
 moveto(3, 103);		/* Line 2-11 */
 outtext("CW Speed Faster      Alt F8   |   Spectr Disp Down 1 dB   Alt 2");
 moveto(3, 113);		/* Line 2-12 */
 outtext("AR                      '['   |   Spectr Display Up 1 dB  Alt 3");
 moveto(3,123);			/* Line 2-13 */
 outtext("SK                      ']'   |   Input RF/Audio       Alt I, i");
 moveto(3,133);			/* Line 2-14 */
 outtext("AS                      '\\'   |");
 moveto(3,143);			/* Line 2-15 */
 outtext("CW Repeat Delimiter     '#'   |   Spectrum to Disk     Alt F, f");
 moveto(3, 153);		/* Line 2-16 */
 outtext("CW Repeat Break         '@'   |   Mode=cw, usb, lsb... Alt M, m");
 moveto(3, 163);		/* Line 2-17 */
 outtext("2 Sec On/Off    '{' and '}'   |");
 moveto(3, 173);		/* Line 2-18 */
 outtext("        BAND CHANGE           |   Require scroll lock:");
 moveto(3, 183);		/* Line 2-19 */
 outtext("2-meters   Ctrl-Alt 1,2...5   |   Xmit Power down 1 dB   Scrl O");
 moveto(3, 193);		/* Line 2-20 */
 outtext("50 MHz Trnsvrtr  Ctrl-Alt A   |   Xmit Power Up 1 dB     Scrl P");
 moveto(3, 203);		/* Line 2-21 */
 outtext("144:B  222:C  432:D   903:9   |   Change Diagnostics    Scrl F3");
 moveto(3, 213);		/* Line 2-22 */
 outtext("1296:E  2304:F ... 241GHz:O   |   Exit to DOS       Scrl-Alt F4");

	for(;;)			/* Wait for keyboard hit */
	    if ( kbhit() )
		{ _bios_keybrd(_KEYBRD_READ);  break; }
	clearviewport();
	line(1, 0, 509, 0);
	line(1, 223, 509, 223);
 /* 63 characters:
          123456789012345678901234567890|23456789012345678901234567890123 */
 moveto(3, 3);			/* Line 3-1 */
 outtext("             DSP-10 KEYBOARD COMMANDS - Screen 3");
 moveto(3, 13);			/* Line 3-2 */
 outtext("    SPECTRAL WATERFALL                        OTHER");
 moveto(3, 23);			/* Line 3-3 */
 outtext("Brightness Down          F5   |   Require Scroll Lock on:");
 moveto(3, 33);			/* Line 3-4 */
 outtext("Brightness Up            F6   |   Disp Dot Type         Scrl F5");
 moveto(3, 43);			/* Line 3-5 */
 outtext("Contrast Down         Sh F5   |   Spectral Data 8/16    Scrl F2");
 moveto(3, 53);			/* Line 3-6 */
 outtext("Contrast Up           Sh F6   |   Wrm Boot DSP Scrl-Ctrl-Alt F4");
 moveto(3, 63);			/* Line 3-7 */
 outtext("");
 moveto(3, 73);			/* Line 3-8 */
 sprintf(cc,
         "Change Location '<' and '>'   |   Fdoppler=%0.1lf Hz", f_eme);
 outtext(cc);
 moveto(3, 83);			/* Line 3-9 */
 outtext("             -----  MOON/SUN LOCATIONS  -----");
 for(i=0; i<=9; i++)
    {
    if (loc[i].time_zone != 999)
	{
	sprintf(cc, "%d %6.6s %31.31s %10.5lf %10.5lf",
	    i,
	    left_justify(loc[i].nm, cc1, 6, SPACE),
	    left_justify(loc[i].name, cc2, 31, HYPHEN),
	    r2d(loc[i].lat), r2d(loc[i].lon));
	moveto(3, 93+10*i);
	outtext(cc);
	}
    else
	{
	sprintf(cc, "%d", i);
	moveto(3, 93+10*i);
	outtext(cc);
	}

    }
 moveto(3, 193);		/* Line 3-20 */
 outtext("");
 moveto(3, 203);		/* Line 3-21 */
 outtext("       DSP-10 Program by W7PUA - boblark@proaxis.com");
 moveto(3, 213);		/* Line 3-22 */
 outtext("       Program Copyright (c) Robert Larkin 1996-2001");

	for(;;)			/* Wait for keyboard hit */
	    if ( kbhit() )
		{_bios_keybrd(_KEYBRD_READ);  break; }
	setviewport(X2L+1, Y2T+96, X2R-1, Y2B-12, DO_CLIP);
	clearviewport();
	putimage(0,   0, big_area_ptr1, COPY_PUT);
	putimage(0, 161, big_area_ptr2, COPY_PUT);
	}

/* left_justify(char *str1, char *str2, int len2, char fill)
 * str1 is copied to str2 and the space to the right is filled with spaces.
 * Therefore str1 should not be longer than len2 or it will be truncated.
 * The difference between str1 and str2 is right-filled with 'fill'.
 * str2 is terminated with a null and must be declared as len2+1 or more.
 */
char *left_justify(char *str1, char *str2, int len2, char fill)
    {
    int len1, l, i;

    len1=strlen(str1);
    if (len1>len2)			/* Won't fit case!  */
	l=len2;
    else
	{
	l=len1;
	for (i=len1; i<len2; i++)	/* Fill to end with blanks */
	    *(str2+i)=fill;
	}
    for (i=0; i<l; i++)			/* Move the string into place */
	*(str2+i)=*(str1+i);
    if (l < len2)
	*(str2+l)=32;			/* Single blank at end; readability */
    *(str2+len2)=0;			/* Terminate the string in a null */
    return str2;
    }


void delayu(unsigned int ncount)
	{unsigned int i;
	 for (i=0; i<ncount; i++);
	}

/* Alternate error handlers  so that program does not exit to dos from
   Ctrl-C or from floating point error */
void  ctrlc_handler(int sig)
	{
	do_t_error(45, "W45 Ctrl-C", 0);
	signal(SIGINT, ctrlc_handler);  /* Reinstall for next time */
	}

void  fpe_handler(int sig)
	{
	errcode=_clear87();
	do_t_error(46,"E46 Float Point", 0);
	signal(SIGFPE, fpe_handler);
	}

void display_summary(char *fname, char *ver)
	{
	char  *pc;

	strcpy(cc, fname);		/* Get a copy to play with */
	cc[strlen(cc)-4]=0;		/* Remove the .exe on the end */
	pc=&cc[strlen(cc)];		/* Now look for end of path */
	while (*pc!='\\' && pc!=&cc[0])  pc--;
	pc++;				/* Move past the slash */
	printf("\nUHFA Transceiver Controller v%s  de W7PUA\n\n", ver);
	printf("Your command line lacked necessary information.\n");
	printf("Use '%s com' where com is a number between 1 and\n", pc);
	printf("4 that sets the COM port to be used.\n");
	printf("Thus for COM2 enter\n");
	printf("        %s 2 \n\n", pc);
	}

void license_info(void)
    {
    clrscr();
    print_stars();
    printf("\n\n\t     2-Meter Transceiver DSP-10 (uhfa.exe) v2.02\n\n");
    printf("\t\t\t           2 May 2001\n\n");
    printf("\t\tCopyright (c) 1996-2001 Robert S. Larkin\n\n");
    print_stars();  printf("\n\n");

 printf("\nDSP-10 (uhfa.exe) comes with ABSOLUTELY NO WARRANTY; for details type `1'.");
 printf("\nThis is free software, and you are welcome to redistribute it");
 printf("\nunder certain conditions; type `2' for details, or alternatively, read");
 printf("\nthe plain text file GNU_GPL.txt that is in the same directory with");
 printf("\nthis executable file uhfa.exe. Be sure to read it.\n");
 printf("\nHit the 'Enter' key (Return key) to continue with the DSP-10 program.");
    }

void warranty(void)
    {
    clrscr();
 printf("\n\n                 NO WARRANTY\n");
 printf("\nBECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY");
 printf("\nFOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN");
 printf("\nOTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES");
 printf("\nPROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED");
 printf("\nOR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF");
 printf("\nMERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS");
 printf("\nTO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE");
 printf("\nPROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,");
 printf("\nREPAIR OR CORRECTION.\n");
 printf("\nIN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING");
 printf("\nWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR");
 printf("\nREDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,");
 printf("\nINCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING");
 printf("\nOUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED");
 printf("\nTO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY");
 printf("\nYOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER");
 printf("\nPROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE");
 printf("\nPOSSIBILITY OF SUCH DAMAGES.\n");
 printf("\nAfter reading, hit Enter (Return) to continue");
    getch();
    }

void license(void)
    {
    char cline[90];
    int nl;

    nl=0;
    clrscr();
    if (NULL == ( license_file=fopen("GNU_GPL.txt", "r") ))
	    {
	    printf("\nNo GNU_GPL.txt license file was found. This must be in");
	    printf(" the same directory\nas UHFA.EXE file. Exiting program.");
	    exit(1);
	    }
    while (fgets(cline, 80, license_file))
	{
	printf("%s", cline);
	if (nl++ >= 20)
	    {
	    nl=0;
	    printf("\n\n(After reading, hit any key to continue)");
	    getch();
	    clrscr();
	    }
	}
    fclose(license_file);
    printf("\n\n(After reading, hit any key to continue)");
    getch();
    clrscr();
    }

void quit(void)
	{
	free( big_area_ptr1 );
	free( big_area_ptr2 );
	free( power10 );
	free( pwrdat );
	free( dnorm );
	free( lt_dnorm );
	free( lt_pwrdat );
	free( pd16 );
	pua43_update(0, -2);		/* De allocate memory */
	if (transmit)  t_r_toggle();  /* Leave hardware in rcv */
	if (quit_save_state)
	    save_state();
	ASYNCSTOP();
	setallpalette(&savpalette);	/* The palette when we got here */
	setbkcolor(BLACK);
	setcolor(WHITE);
	closegraph();
	printf("Exiting through quit()\n");
	exit(0);
	}
